@@ -49,6 +49,13 @@ class Slice(NDIndex):
     >>> s.raw
     slice(None, 10, None)
 
+    For most use-cases, it's more convenient to create Slice objects using
+    `ndindex[slice]`, which allows using `a:b` slicing syntax:
+
+    >>> from ndindex import ndindex
+    >>> ndindex[0:10]
+    Slice(0, 10, None)
+
     """
     __slots__ = ()
 
@@ -76,9 +83,11 @@ class Slice(NDIndex):
         return args
 
     def __hash__(self):
-        # We can't use the default hash(self.raw) because slices are not
-        # hashable
-        return hash(self.args)
+        # Slices are only hashable in Python 3.12+
+        try:
+            return hash(self.raw)
+        except TypeError: # pragma: no cover
+            return hash(self.args)
 
     @property
     def raw(self):
@@ -200,7 +209,7 @@ class Slice(NDIndex):
 
         return len(range(start, stop, step))
 
-    def reduce(self, shape=None, axis=0):
+    def reduce(self, shape=None, *, axis=0, negative_int=False):
         """
         `Slice.reduce` returns a slice that is canonicalized for an array of the
         given shape, or for any shape if `shape` is `None` (the default).
@@ -574,6 +583,14 @@ class Slice(NDIndex):
             return self.args == other.args
         return False
 
+    def selected_indices(self, shape, axis=None):
+        if axis is None:
+            yield from self.expand(shape).selected_indices(shape)
+        else:
+            shape = asshape(shape, axis=axis)
+            for i in range(shape[axis])[self.raw]:
+                yield Integer(i)
+
 # Imports at the bottom to avoid circular import issues
 from .ndindex import ndindex
 from .tuple import Tuple
